"""
Assignment:
Create a function db_query as specified by doc and doctesting unit-tests below.
"""

from csv import DictReader, DictWriter
from pathlib import Path


def db_query(qry):
    """
    Executes a query over the records stored in CSV files in "data" directory. Works as a generator which iterates
    over records for which the "qry" function returns True. Each record is represented as a dict (keyed by the
    column names as in the CSV files). The files are taken in alphabetical order over their full paths relative to the
    "data" directory (e.g. "01/01.csv", "01/02.csv", "02/01.csv", ...).
    :param qry: Function which returns True if a given record should be returned by the generator. If the function fails
        with an exception, the exception is propagated from the db_query
    :return: Generator over matching records


    >>> list(db_query(lambda x: True))
    [{'First Name': 'Derek', 'Last Name': 'Holmes', 'Gender': 'Male', 'Age': '27', 'Email': 'd.holmes@randatmail.com', 'Salary': '178973', 'Marital Status': 'Single', 'Number of Children': '0'}, {'First Name': 'April', 'Last Name': 'Edwards', 'Gender': 'Female', 'Age': '27', 'Email': 'a.edwards@randatmail.com', 'Salary': '41135', 'Marital Status': 'Single', 'Number of Children': '3'}, {'First Name': 'Paige', 'Last Name': 'Campbell', 'Gender': 'Female', 'Age': '29', 'Email': 'p.campbell@randatmail.com', 'Salary': '44298', 'Marital Status': 'Single', 'Number of Children': '1'}, {'First Name': 'James', 'Last Name': 'Richardson', 'Gender': 'Male', 'Age': '26', 'Email': 'j.richardson@randatmail.com', 'Salary': '58407', 'Marital Status': 'Single', 'Number of Children': '1'}, {'First Name': 'Honey', 'Last Name': 'Riley', 'Gender': 'Female', 'Age': '29', 'Email': 'h.riley@randatmail.com', 'Salary': '54028', 'Marital Status': 'Single', 'Number of Children': '4'}, {'First Name': 'Sienna', 'Last Name': 'Perkins', 'Gender': 'Female', 'Age': '30', 'Email': 's.perkins@randatmail.com', 'Salary': '192662', 'Marital Status': 'Single', 'Number of Children': '0'}, {'First Name': 'Max', 'Last Name': 'Higgins', 'Gender': 'Male', 'Age': '20', 'Email': 'm.higgins@randatmail.com', 'Salary': '141911', 'Marital Status': 'Single', 'Number of Children': '0'}, {'First Name': 'Briony', 'Last Name': 'Gray', 'Gender': 'Female', 'Age': '22', 'Email': 'b.gray@randatmail.com', 'Salary': '98443', 'Marital Status': 'Single', 'Number of Children': '0'}, {'First Name': 'Alina', 'Last Name': 'Baker', 'Gender': 'Female', 'Age': '20', 'Email': 'a.baker@randatmail.com', 'Salary': '72180', 'Marital Status': 'Married', 'Number of Children': '1'}, {'First Name': 'Kimberly', 'Last Name': 'Fowler', 'Gender': 'Female', 'Age': '30', 'Email': 'k.fowler@randatmail.com', 'Salary': '153785', 'Marital Status': 'Single', 'Number of Children': '4'}, {'First Name': 'Olivia', 'Last Name': 'Ross', 'Gender': 'Female', 'Age': '29', 'Email': 'o.ross@randatmail.com', 'Salary': '72566', 'Marital Status': 'Single', 'Number of Children': '0'}, {'First Name': 'George', 'Last Name': 'Ross', 'Gender': 'Male', 'Age': '19', 'Email': 'g.ross@randatmail.com', 'Salary': '53454', 'Marital Status': 'Single', 'Number of Children': '3'}, {'First Name': 'Jasmine', 'Last Name': 'Evans', 'Gender': 'Female', 'Age': '27', 'Email': 'j.evans@randatmail.com', 'Salary': '88449', 'Marital Status': 'Married', 'Number of Children': '5'}, {'First Name': 'Sam', 'Last Name': 'Chapman', 'Gender': 'Male', 'Age': '30', 'Email': 's.chapman@randatmail.com', 'Salary': '85806', 'Marital Status': 'Single', 'Number of Children': '3'}, {'First Name': 'Michelle', 'Last Name': 'Richardson', 'Gender': 'Female', 'Age': '26', 'Email': 'm.richardson@randatmail.com', 'Salary': '42532', 'Marital Status': 'Married', 'Number of Children': '3'}, {'First Name': 'Mike', 'Last Name': 'Richards', 'Gender': 'Male', 'Age': '20', 'Email': 'm.richards@randatmail.com', 'Salary': '158756', 'Marital Status': 'Single', 'Number of Children': '4'}, {'First Name': 'Paul', 'Last Name': 'Barnes', 'Gender': 'Male', 'Age': '24', 'Email': 'p.barnes@randatmail.com', 'Salary': '136052', 'Marital Status': 'Married', 'Number of Children': '5'}, {'First Name': 'Carl', 'Last Name': 'Clark', 'Gender': 'Male', 'Age': '22', 'Email': 'c.clark@randatmail.com', 'Salary': '59129', 'Marital Status': 'Married', 'Number of Children': '5'}, {'First Name': 'Sawyer', 'Last Name': 'Miller', 'Gender': 'Male', 'Age': '23', 'Email': 's.miller@randatmail.com', 'Salary': '38453', 'Marital Status': 'Married', 'Number of Children': '0'}, {'First Name': 'Kelvin', 'Last Name': 'Nelson', 'Gender': 'Male', 'Age': '25', 'Email': 'k.nelson@randatmail.com', 'Salary': '64450', 'Marital Status': 'Married', 'Number of Children': '1'}, {'First Name': 'Ada', 'Last Name': 'Gibson', 'Gender': 'Female', 'Age': '20', 'Email': 'a.gibson@randatmail.com', 'Salary': '134315', 'Marital Status': 'Single', 'Number of Children': '4'}, {'First Name': 'Dexter', 'Last Name': 'Ferguson', 'Gender': 'Male', 'Age': '22', 'Email': 'd.ferguson@randatmail.com', 'Salary': '171611', 'Marital Status': 'Married', 'Number of Children': '2'}, {'First Name': 'Andrew', 'Last Name': 'Stewart', 'Gender': 'Male', 'Age': '19', 'Email': 'a.stewart@randatmail.com', 'Salary': '185298', 'Marital Status': 'Married', 'Number of Children': '3'}, {'First Name': 'Adam', 'Last Name': 'Higgins', 'Gender': 'Male', 'Age': '30', 'Email': 'a.higgins@randatmail.com', 'Salary': '181786', 'Marital Status': 'Married', 'Number of Children': '5'}, {'First Name': 'Lucy', 'Last Name': 'Crawford', 'Gender': 'Female', 'Age': '25', 'Email': 'l.crawford@randatmail.com', 'Salary': '123337', 'Marital Status': 'Married', 'Number of Children': '5'}, {'First Name': 'Blake', 'Last Name': 'Gibson', 'Gender': 'Male', 'Age': '19', 'Email': 'b.gibson@randatmail.com', 'Salary': '156362', 'Marital Status': 'Married', 'Number of Children': '4'}]

    >>> list(db_query(lambda x: False))
    []

    >>> list(db_query(lambda x: x['Email'].startswith('l')))
    [{'First Name': 'Lucy', 'Last Name': 'Crawford', 'Gender': 'Female', 'Age': '25', 'Email': 'l.crawford@randatmail.com', 'Salary': '123337', 'Marital Status': 'Married', 'Number of Children': '5'}]

    >>> list(db_query(lambda x: int(x['Age']) >= 25 and x['Marital Status'] == 'Single'))
    [{'First Name': 'Derek', 'Last Name': 'Holmes', 'Gender': 'Male', 'Age': '27', 'Email': 'd.holmes@randatmail.com', 'Salary': '178973', 'Marital Status': 'Single', 'Number of Children': '0'}, {'First Name': 'April', 'Last Name': 'Edwards', 'Gender': 'Female', 'Age': '27', 'Email': 'a.edwards@randatmail.com', 'Salary': '41135', 'Marital Status': 'Single', 'Number of Children': '3'}, {'First Name': 'Paige', 'Last Name': 'Campbell', 'Gender': 'Female', 'Age': '29', 'Email': 'p.campbell@randatmail.com', 'Salary': '44298', 'Marital Status': 'Single', 'Number of Children': '1'}, {'First Name': 'James', 'Last Name': 'Richardson', 'Gender': 'Male', 'Age': '26', 'Email': 'j.richardson@randatmail.com', 'Salary': '58407', 'Marital Status': 'Single', 'Number of Children': '1'}, {'First Name': 'Honey', 'Last Name': 'Riley', 'Gender': 'Female', 'Age': '29', 'Email': 'h.riley@randatmail.com', 'Salary': '54028', 'Marital Status': 'Single', 'Number of Children': '4'}, {'First Name': 'Sienna', 'Last Name': 'Perkins', 'Gender': 'Female', 'Age': '30', 'Email': 's.perkins@randatmail.com', 'Salary': '192662', 'Marital Status': 'Single', 'Number of Children': '0'}, {'First Name': 'Kimberly', 'Last Name': 'Fowler', 'Gender': 'Female', 'Age': '30', 'Email': 'k.fowler@randatmail.com', 'Salary': '153785', 'Marital Status': 'Single', 'Number of Children': '4'}, {'First Name': 'Olivia', 'Last Name': 'Ross', 'Gender': 'Female', 'Age': '29', 'Email': 'o.ross@randatmail.com', 'Salary': '72566', 'Marital Status': 'Single', 'Number of Children': '0'}, {'First Name': 'Sam', 'Last Name': 'Chapman', 'Gender': 'Male', 'Age': '30', 'Email': 's.chapman@randatmail.com', 'Salary': '85806', 'Marital Status': 'Single', 'Number of Children': '3'}]

    >>> list(db_query(lambda x: x['XXX'] == 'foo'))
    Traceback (most recent call last):
    ...
    KeyError: 'XXX'

    >>> list(db_query(lambda x: x['Email'].startswith('a')))
    [{'First Name': 'April', 'Last Name': 'Edwards', 'Gender': 'Female', 'Age': '27', 'Email': 'a.edwards@randatmail.com', 'Salary': '41135', 'Marital Status': 'Single', 'Number of Children': '3'}, {'First Name': 'Alina', 'Last Name': 'Baker', 'Gender': 'Female', 'Age': '20', 'Email': 'a.baker@randatmail.com', 'Salary': '72180', 'Marital Status': 'Married', 'Number of Children': '1'}, {'First Name': 'Ada', 'Last Name': 'Gibson', 'Gender': 'Female', 'Age': '20', 'Email': 'a.gibson@randatmail.com', 'Salary': '134315', 'Marital Status': 'Single', 'Number of Children': '4'}, {'First Name': 'Andrew', 'Last Name': 'Stewart', 'Gender': 'Male', 'Age': '19', 'Email': 'a.stewart@randatmail.com', 'Salary': '185298', 'Marital Status': 'Married', 'Number of Children': '3'}, {'First Name': 'Adam', 'Last Name': 'Higgins', 'Gender': 'Male', 'Age': '30', 'Email': 'a.higgins@randatmail.com', 'Salary': '181786', 'Marital Status': 'Married', 'Number of Children': '5'}]
    """

    for filename in sorted(Path('data').glob('**/*.csv')):
        with open(filename) as file:
            reader = DictReader(file)

            for record in reader:
                if qry(record):
                    yield dict(record)

